/*
 * sertrans.c - serial transfer
 *
 * Written by
 *  Christian Vogelgsang <chris@vogelgsang.org>
 *
 * This file is part of dtv2ser.
 * See README for copyright notice.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

#include "global.h"
#include "uart.h"
#include "sertrans.h"
#include "util.h"

static u08 serial_begin_transfer(u32 length)
{ return TRANSFER_OK; }

static u08 serial_end_transfer(u08 lastStatus)
{ return lastStatus; }

static u08 serial_begin_block(u16 length)
{ 
  // send u16 block length
  u08 data = length >> 8;
  if(!uart_send_with_rts(data))
    return TRANSFER_ERROR_CLIENT_TIMEOUT;
  data = length & 0xff;
  if(!uart_send_with_rts(data))
    return TRANSFER_ERROR_CLIENT_TIMEOUT;

  return TRANSFER_OK; 
}

static u08 serial_end_block(u08 lastStatus,u16 crc16)
{ 
  // send u16 crc16
  u08 data = crc16 >> 8;
  if(!uart_send_with_rts(data))
    return TRANSFER_ERROR_CLIENT_TIMEOUT;
  data = crc16 & 0xff;
  if(!uart_send_with_rts(data))
    return TRANSFER_ERROR_CLIENT_TIMEOUT;
  
  // send last status to client
  if(!uart_send_with_rts(lastStatus))
    return TRANSFER_ERROR_CLIENT_TIMEOUT;
  
  return lastStatus;
}

static u08 serial_read_byte(u08 *data)
{ 
  if(uart_read_with_cts(data))
    return TRANSFER_OK;
  else
    return TRANSFER_ERROR_CLIENT_TIMEOUT; 
}

static u08 serial_write_byte(u08 *data)
{  
  if(uart_send_with_rts(*data))
    return TRANSFER_OK;
  else
    return TRANSFER_ERROR_CLIENT_TIMEOUT;
}

host_transfer_funcs_t serial_host_read_funcs =
{
  .begin_transfer = serial_begin_transfer,
  .end_transfer   = serial_end_transfer,
  .begin_block    = serial_begin_block,
  .end_block      = serial_end_block,
  .transfer_byte  = serial_read_byte
};

host_transfer_funcs_t serial_host_write_funcs =
{
  .begin_transfer = serial_begin_transfer,
  .end_transfer   = serial_end_transfer,
  .begin_block    = serial_begin_block,
  .end_block      = serial_end_block,
  .transfer_byte  = serial_write_byte
};

